"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const _utils_1 = require('../../utils');
class Cpp {
    constructor(rootDir, execPath, includeDirectories) {
        this.rootDir = rootDir;
        this.execPath = execPath;
        this.includeDirectories = includeDirectories;
        this.content = '';
        this.includes = new Set();
        this.stdIncludes = new Set();
    }
    buildCpp(sourceFile) {
        return __awaiter(this, void 0, void 0, function* () {
            this.content = '';
            this.includes.clear();
            this.stdIncludes.clear();
            sourceFile = path.resolve(this.execPath, sourceFile);
            yield this.parseSourceFiles(sourceFile);
            let stdIncludes = [...this.stdIncludes].map(item => `#include<${item}>`).join('\n');
            let content = stdIncludes + '\n' + this.content;
            return content;
        });
    }
    parseSourceFiles(sourceFile) {
        return __awaiter(this, void 0, void 0, function* () {
            _utils_1.logger.debug('load source:', sourceFile);
            sourceFile = path.resolve(this.rootDir, sourceFile);
            if (!(yield _utils_1.default.isFile(sourceFile)))
                return;
            let content = yield fs.readFile(sourceFile, 'utf-8');
            let items = content.split(/\n+/g);
            let cc = '';
            for (let item of items) {
                let result = Cpp.includeRegex.exec(item);
                if (result) {
                    let [, includeItem] = result;
                    if (this.includes.has(includeItem))
                        continue;
                    this.includes.add(includeItem);
                    let target = yield this.generateRealReferPath(includeItem);
                    if (target)
                        yield this.parseSourceFiles(target);
                    else
                        this.stdIncludes.add(includeItem);
                }
                else
                    cc += item + '\n';
            }
            this.content += '\n' + cc;
        });
    }
    generateRealReferPath(includeItem) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let dir of this.includeDirectories) {
                let p = path.resolve(this.rootDir, dir, includeItem);
                if (yield _utils_1.default.isFile(p))
                    return p;
            }
            return null;
        });
    }
}
Cpp.includeRegex = /^#include\s*<(\S+)>\s*?/;
exports.Cpp = Cpp;
