"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const cp = require("copy-paste");
const emm_logger_1 = require("emm-logger");
const _utils_1 = require('../../utils');
const cpp_1 = require("./cpp");
const handlers_1 = require("./handlers");
const logger = new emm_logger_1.EmmLogger('build');
exports.default = (cmdArgs) => __awaiter(this, void 0, void 0, function* () {
    try {
        yield preproccess(cmdArgs);
    }
    catch (error) {
        logger.error('preproccess cmdArgs failed.');
        logger.error(error);
        process.exit(-1);
    }
    let { rootDir, execPath, cmakeLists, options, } = cmdArgs;
    let { target, outputPath } = options;
    let { include_directories } = yield _utils_1.parseCMakeLists(cmakeLists);
    logger.debug('include_directories:', include_directories);
    let cpp = new cpp_1.Cpp(rootDir, execPath, include_directories);
    let content = yield cpp.buildCpp(target);
    let { flags } = options;
    if (flags.uglify) {
        flags.removeComments = true;
        flags.removeSpaces = true;
    }
    if (flags.removeComments) {
        logger.info('removing comment.');
        content = handlers_1.removeComments(content);
    }
    if (flags.removeSpaces) {
        logger.info('removing spaces.');
        content = handlers_1.removeSpaces(content);
    }
    if (!_utils_1.default.isNil(outputPath)) {
        logger.info(`write \`${outputPath}\``);
        yield fs.writeFile(outputPath, content, 'utf-8');
    }
    else {
        logger.info(`copying to system-clip.`);
        yield new Promise((resolve) => {
            cp.copy(content, resolve);
        });
    }
});
function preproccess(cmdArgs) {
    return __awaiter(this, void 0, void 0, function* () {
        let { execPath, options } = cmdArgs;
        let { out, dir, path: p, target } = options;
        let flag = false;
        if (p) {
            p = path.resolve(execPath, p);
            logger.debug(`indexed --path \`${p}\`.`);
            if (!(yield _utils_1.default.isFile(p)))
                throw new Error(`${p} isn't a valid file path.`);
            dir = path.dirname(p);
            out = path.basename(p);
            flag = true;
        }
        else {
            if (!_utils_1.default.isNil(out)) {
                flag = true;
                if (!dir)
                    dir = execPath;
                dir = path.resolve(execPath, dir);
                if (!_utils_1.default.isString(out)) {
                    let { name, ext } = path.parse(target);
                    out = `${name}.out${ext}`;
                }
                if (!(yield _utils_1.default.isDirectory(dir)))
                    throw new Error(`${dir} isn't a valid directory.`);
            }
        }
        let { flags } = options;
        cmdArgs.options = {
            flags,
            target,
            outputPath: flag ? path.resolve(dir, out) : undefined,
        };
    });
}
