"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const program = require("commander");
const _utils_1 = require('../utils');
const build_1 = require("./build");
const new_1 = require("./new");
exports.default = (version, args) => __awaiter(this, void 0, void 0, function* () {
    program
        .version(version)
        .option('--log-level <level>')
        .option('--log-option <option>')
        .option('--cl, --cmake-lists <cmakeLists-name>', 'index cmakeLists name.');
    program
        .command(`build <target>`)
        .option('-o, --out [output-filename]', 'index output filename')
        .option('-d, --dir <output-directory>', 'index output directory')
        .option('-p, --path <output-path>', 'index output path. It\'s a shortcut of `-o` and `-f`')
        .option('--rc, --remove-comments', 'remove comments.')
        .option('--rs, --remove-spaces', 'remove spaces.')
        .option('-u, --uglify', 'shortcut of --rc --rs.')
        .action((target, options) => {
        (() => __awaiter(this, void 0, void 0, function* () {
            let cmdArgs = yield generateGlobalOptions();
            cmdArgs.options = {
                target: target,
                out: options.out,
                dir: options.dir,
                path: options.path,
                flags: {
                    uglify: options.uglify,
                    removeComments: options.removeComments,
                    removeSpaces: options.removeSpaces,
                }
            };
            _utils_1.logger.debug('cmdArgs:', cmdArgs);
            yield build_1.default(cmdArgs);
        }))();
    });
    program
        .command(`new <targets...>`)
        .option('-t, --template <template-filename>', 'index template file path(the path to the current folder or absolute path')
        .action((targets, options) => {
        (() => __awaiter(this, void 0, void 0, function* () {
            let cmdArgs = yield generateGlobalOptions();
            cmdArgs.options = {
                targets: targets,
                template: options.template,
            };
            _utils_1.logger.debug('cmdArgs:', cmdArgs);
            yield new_1.default(cmdArgs);
        }))();
    });
    program
        .parse(args);
    function generateGlobalOptions() {
        return __awaiter(this, void 0, void 0, function* () {
            let execPath = path.resolve();
            let cl = program.cmakeLists || 'CMakeLists.txt';
            let cmakeLists = yield _utils_1.default.findNearestTarget(execPath, cl);
            if (!(yield _utils_1.default.isFile(cmakeLists))) {
                _utils_1.logger.fatal(`\`${cl}\` is not exists.`);
                process.exit(-1);
            }
            let rootDir = path.dirname(cmakeLists);
            _utils_1.logger.debug('rootDir:', rootDir);
            _utils_1.logger.debug('execPath:', execPath);
            _utils_1.logger.debug('cmakeLists:', cmakeLists);
            return {
                rootDir,
                execPath,
                cmakeLists,
            };
        });
    }
});
