"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
const emm_logger_1 = require("emm-logger");
const _utils_1 = require('../../utils');
const logger = new emm_logger_1.EmmLogger('new', {
    show_date: false,
});
exports.default = (cmdArgs) => __awaiter(this, void 0, void 0, function* () {
    let { rootDir, execPath, cmakeLists, options } = cmdArgs;
    let { targets, template } = options;
    let { add_executables } = yield _utils_1.parseCMakeLists(cmakeLists);
    if (template)
        template = path.resolve(execPath, template);
    else {
        template = path.resolve(rootDir, 'template.cpp');
        logger.info(`set template-path as \`${template}\` to default.`);
    }
    for (let target of targets) {
        if (!path.extname(target))
            target += '.cpp';
        yield addTarget(target);
    }
    function addTarget(target) {
        return __awaiter(this, void 0, void 0, function* () {
            target = path.resolve(execPath, target);
            let targetName = path.relative(rootDir, target).replace(/\\/g, '/');
            let name = _utils_1.generateExecutableTargetName(add_executables, targetName);
            if (!add_executables.has(name)) {
                let add_executable = `add_executable(${name} ${targetName})`;
                logger.info('add executable:', add_executable);
                yield fs.appendFile(cmakeLists, add_executable + '\n');
            }
            else {
                logger.warn(`there has exists \`add_executable(${name} ${add_executables.get(name)})\`.`);
            }
            if (!(yield _utils_1.default.isFile(target))) {
                logger.info(`creating file \`${target}\`.`);
                yield fs.writeFile(target, '', 'utf-8');
                if (template && (yield _utils_1.default.isFile(template))) {
                    let content = yield fs.readFile(template, 'utf-8');
                    logger.info(`write template into \`${target}\`.`);
                    yield fs.writeFile(target, content, 'utf-8');
                }
            }
            else {
                logger.warn(`\`${target}\` has existed.`);
            }
        });
    }
});
