"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs-extra");
const path = require("path");
exports.include_directories_regex = /^\s*include_directories\((\s*\S+?\s*)\)\s*$/;
exports.add_executable_regex = /^\s*add_executable\((\S+?)(\s+[\s\S]+?)\)\s*$/;
function parseCMakeLists(cmakeLists) {
    return __awaiter(this, void 0, void 0, function* () {
        let content = yield fs.readFile(cmakeLists, 'utf-8');
        let items = content.split(/\n+/g);
        let include_directories = [];
        for (let item of items) {
            let result = exports.include_directories_regex.exec(item);
            if (result) {
                let [, include_directory] = result;
                include_directories.push(include_directory);
            }
        }
        let add_executables = new Map();
        for (let item of items) {
            let result = exports.add_executable_regex.exec(item);
            if (result) {
                let [, target, sources] = result;
                add_executables.set(target, sources);
            }
        }
        return {
            include_directories,
            add_executables,
        };
    });
}
exports.parseCMakeLists = parseCMakeLists;
function generateExecutableTargetName(add_executable, source) {
    let { dir: dirname, name: filename } = path.parse(source);
    let name = dirname.split(/[\\/]/g).map(m => m[0].toUpperCase()).join('');
    name = 'P' + name + filename;
    return name;
}
exports.generateExecutableTargetName = generateExecutableTargetName;
