"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function splitContentToQuotedItems(content) {
    const quotedRegex = new RegExp(generateQuotedRegexSource([`'`, `"`]), 'g');
    let quotedItems = [];
    for (let p; p = quotedRegex.exec(content);) {
        let [target] = p;
        quotedItems.push(target);
    }
    let nonQuotedItems = content.split(quotedRegex);
    return {
        validItems: quotedItems,
        invalidItems: nonQuotedItems,
    };
}
exports.splitContentToQuotedItems = splitContentToQuotedItems;
function splitContentToCommentItems(content, blockComment, lineComment) {
    const quotedRegex = new RegExp(`^${generateQuotedRegexSource([`'`, `"`])}`);
    const commentRegex = new RegExp(`^${generateCommentRegexSource(blockComment, lineComment)}`);
    let commentItems = [];
    let nonCommentItems = [];
    let lastId = 0;
    for (let i = 0; i < content.length;) {
        if (content[i] === `'` || content[i] === `"`) {
            let matched = quotedRegex.exec(content.slice(i));
            if (matched) {
                let [quoteContent] = matched;
                i += quoteContent.length;
            }
            else
                throw new Error(`invalid quoted.`);
        }
        else if (isPrefixOf(content, i, blockComment[0]) || isPrefixOf(content, i, lineComment)) {
            nonCommentItems.push(content.slice(lastId, i));
            let matched = commentRegex.exec(content.slice(i));
            if (matched) {
                let [commentContent] = matched;
                lastId = i + commentContent.length;
                commentItems.push(content.slice(i, lastId));
                i = lastId;
            }
            else
                throw new Error(`invalid comments.`);
        }
        else
            ++i;
    }
    nonCommentItems.push(content.slice(lastId));
    return {
        validItems: commentItems,
        invalidItems: nonCommentItems,
    };
}
exports.splitContentToCommentItems = splitContentToCommentItems;
function isPrefixOf(content, pos, prefix) {
    if (content.length - pos < prefix.length)
        return false;
    for (let i = 0; i < prefix.length; ++i)
        if (content[pos + i] !== prefix[i])
            return false;
    return true;
}
exports.isPrefixOf = isPrefixOf;
function mergeSourceItemsToContent(sourceItems) {
    let { validItems, invalidItems } = sourceItems;
    let content = '';
    for (let i = 0; i < validItems.length; ++i) {
        content += invalidItems[i];
        content += validItems[i];
    }
    content += invalidItems[validItems.length];
    return content;
}
exports.mergeSourceItemsToContent = mergeSourceItemsToContent;
function generateQuotedRegexSource(quotes) {
    let reg_source = '';
    for (let quote of quotes) {
        if (quote.length !== 1)
            throw new Error(`quotes must be an array of character.`);
        reg_source += `${quote}(?:\\\\${quote}|[\\s\\S][^${quote}])*?(?:|[^\\\\])${quote}|`;
    }
    reg_source = reg_source.slice(0, -1);
    return reg_source;
}
exports.generateQuotedRegexSource = generateQuotedRegexSource;
function generateCommentRegexSource(blockComment, lineComment) {
    let [leftBlockCommentMark, rightBlockCommentMark] = blockComment;
    leftBlockCommentMark = toRegexPlainString(leftBlockCommentMark);
    rightBlockCommentMark = toRegexPlainString(rightBlockCommentMark);
    lineComment = toRegexPlainString(lineComment);
    let reg_source = `${leftBlockCommentMark}[\\s\\S]*?${rightBlockCommentMark}|${lineComment}[^\n]*`;
    return reg_source;
}
exports.generateCommentRegexSource = generateCommentRegexSource;
function toRegexPlainString(s) {
    return s.replace(/([.?*+\\()\[\]{}|$])/g, (text, meta) => `\\${meta}`);
}
exports.toRegexPlainString = toRegexPlainString;
